#include "copyright.h"
#include "port.h"
#include "rpc.h"

#define oct_prepend_error(string)

#define MYNIL(type) (type *) 0

char *oct_type_names[] = {
    "OCT_UNDEFINED_OBJECT",
    "OCT_FACET",
    "OCT_TERM",
    "OCT_NET",
    "OCT_INSTANCE",
    "OCT_POLYGON",
    "OCT_BOX",
    "OCT_CIRCLE",
    "OCT_PATH",
    "OCT_LABEL",
    "OCT_PROP",
    "OCT_BAG",
    "OCT_LAYER",
    (char *) 0
};

static char *trans_name[] = {
    "TR_NO_TRANSFORM",
    "TR_MIRROR_X",
    "TR_MIRROR_Y",
    "TR_ROT90",
    "TR_ROT180",
    "TR_ROT270",
    "TR_MX_ROT90",
    "TR_MY_ROT90",
    "TR_FULL_TRANSFORM"
};

octStatus rpcPrintObject();

char *optProgName = "readtest";

int
octReadTest(spot, cmdList, userOptionWord)
struct RPCSpot *spot;
lsList cmdList;
long userOptionWord;
{
    octObject facet, object;
    octStatus status;
    octGenerator gen;

        /* open the facet for reading */

    facet.objectId = spot->facet;
	
    if (octGetById(&facet) < OCT_OK) {
	octError("test example");
	return(RPC_OK);
    }

    if (octInitGenContentsSpecial(&facet, OCT_ALL_MASK, &gen) != OCT_OK) {
	octError("generating geos");
	return(RPC_OK);
    }

        /* for each object in the facet, call printObject on it */

    while ((status = octGenerate(&gen, &object)) == OCT_OK) {
	if (rpcPrintObject(&object) != OCT_OK) {
	    octError("print fail");
	    return(RPC_OK);
	}
    }

        /*
	 * octGenerate either returns OCT_OK (when it still has an object
         * to return) or OCT_GEN_DONE (when everything has been returned).
	 * Anything else means something went wrong
	 */
    
    if (status != OCT_GEN_DONE) {
	octError("geo generator");
	return(RPC_OK);
    }

    VEMMSG("\nEnd of Read Test\n");
    vemPrompt();
    
    return(RPC_OK);
}

octStatus
rpcPrintObject(object)
struct octObject *object;
{
    octGenerator gen;
    octObject sub_object;
    octStatus status;
    octTransformType type;
    char buffer[1024];

    switch(object->type) {
    case OCT_BOX:
	(void) sprintf(buffer, "Box (%d), (%d,%d) (%d,%d)\n", 
	    object->objectId,
	    object->contents.box.lowerLeft.x,
	    object->contents.box.lowerLeft.y,
	    object->contents.box.upperRight.x,
	    object->contents.box.upperRight.y);
	vemMessage(buffer, MSG_DISP);
	break;
    case OCT_NET:
	(void) sprintf(buffer, "Net (%d) with name %s\n",
		object->objectId,
		(object->contents.net.name == MYNIL(char) ?
		 "<NO-NAME>" : object->contents.net.name));
	vemMessage(buffer, MSG_DISP);
	break;
    case OCT_LAYER:
	(void) sprintf(buffer, "Layer (%d) with name %s\n",
		object->objectId,
		(object->contents.layer.name == MYNIL(char) ?
		 "<NO-NAME>" : object->contents.layer.name));
	vemMessage(buffer, MSG_DISP);
	break;
    case OCT_PATH: {
	int32 num_points = 200;
	struct octPoint pt_array[200];
	int retval,i;
	
	(void) sprintf(buffer, "Path (%d) with width %d\n",
		object->objectId, object->contents.path.width);
	vemMessage(buffer, MSG_DISP);
	retval = octGetPoints(object, &num_points, pt_array);
	if (retval == OCT_OK) {
	    (void) sprintf(buffer, "    with %d points: ", num_points);
	    for (i = 0; i < num_points; i++) {
		(void) sprintf(buffer,"%s (%d,%d) ", buffer, pt_array[i]);
	    }
	    (void) sprintf(buffer,"%s\n", buffer);
	    vemMessage(buffer, MSG_DISP);

	} else if (retval == OCT_TOO_SMALL) {
	    (void) sprintf(buffer,"    with %d points (too many to print)\n");
	    vemMessage(buffer, MSG_DISP);
	} else {
	    oct_prepend_error("OctPrintObject: Getting the points on a path");
	    return retval;
	}
	break;
    }
    case OCT_POLYGON: {
	int num_points = 200;
	struct octPoint pt_array[200];
	int retval,i;
	
	(void) sprintf(buffer, "Polygon (%d)\n", object->objectId);
	vemMessage(buffer, MSG_DISP);
	retval = octGetPoints(object, &num_points, pt_array);
	if (retval == OCT_OK) {
	    (void) sprintf(buffer, "    with %d points: ", num_points);
	    for (i = 0; i < num_points; i++) {
		(void) sprintf(buffer,"%s (%d,%d) ", buffer, pt_array[i]);
	    }
	    (void) sprintf(buffer,"%s\n", buffer);
	    vemMessage(buffer, MSG_DISP);
	} else if (retval == OCT_TOO_SMALL) {
	    (void) sprintf(buffer,"    with %d points (too many to print)\n");
	    vemMessage(buffer, MSG_DISP);
	} else {
	  oct_prepend_error("OctPrintObject: Getting the points on a polygon");
	    return retval;
	}
	break;
    }
    case OCT_LABEL:
	(void) sprintf(buffer, "Label (%d) labeled %s\n",
		object->objectId,
		(object->contents.label.label == MYNIL(char) ?
		 "<NO-NAME>" : object->contents.label.label));
	vemMessage(buffer, MSG_DISP);
	break;
    case OCT_CIRCLE:
	(void) sprintf(buffer, "Circle (%d) centered at (%d,%d)\n",
		object->objectId,
		object->contents.circle.center.x,
		object->contents.circle.center.y);
	vemMessage(buffer, MSG_DISP);
	(void) sprintf(buffer, "    from angle %d to %d, and radius %d to %d\n",
		object->contents.circle.startingAngle,
		object->contents.circle.endingAngle,
		object->contents.circle.innerRadius,
		object->contents.circle.outerRadius);
	vemMessage(buffer, MSG_DISP);
	break;
    case OCT_TERM:
	(void) sprintf(buffer, "Term (%d) with name %s on instance %d\n",
		object->objectId,
		(object->contents.term.name == MYNIL(char) ?
		 "<NO-NAME>" : object->contents.term.name),
		object->contents.term.instanceId);
	vemMessage(buffer, MSG_DISP);
	break;
    case OCT_INSTANCE: 
	(void) sprintf(buffer, "Instance (%d) named %s of (%s %s)\n",
	       object->objectId,
		(object->contents.instance.name == MYNIL(char) ?
		 "<NO-NAME>" : object->contents.instance.name),
	       object->contents.instance.master,
	       object->contents.instance.view);
	vemMessage(buffer, MSG_DISP);
	type = object->contents.instance.transform.transformType;
	(void) sprintf(buffer, "    at (%d,%d) with transform %s\n",
	       object->contents.instance.transform.translation.x,
	       object->contents.instance.transform.translation.y,
	       trans_name[(int)type]);
	vemMessage(buffer, MSG_DISP);
	if (type == OCT_FULL_TRANSFORM) {
	    double (*matrix)[2];
	    matrix = object->contents.instance.transform.generalTransform;

	    (void) sprintf(buffer, "    with matrix [%g,%g],[%g,%g]\n",
		   matrix[0][0], matrix[0][1],matrix[1][0],matrix[1][1]);
	    vemMessage(buffer, MSG_DISP);
	}
	break;
    case OCT_PROP:
	(void) sprintf(buffer, "Prop (%d) named %s, value ",
		object->objectId, object->contents.prop.name);
	vemMessage(buffer, MSG_DISP);
	switch (object->contents.prop.type) {
	case OCT_INTEGER :
	    (void) sprintf(buffer,"Integer %d\n",
		    object->contents.prop.value.integer);
	    vemMessage(buffer, MSG_DISP);
	    break;
	case OCT_REAL :
	    (void) sprintf(buffer,"Real %g\n", object->contents.prop.value.real);
	    vemMessage(buffer, MSG_DISP);
	    break;
	case OCT_STRING :
	    (void) sprintf(buffer,"String %s\n", object->contents.prop.value.string);
	    vemMessage(buffer, MSG_DISP);
	    break;
	case OCT_STRANGER :
	    {
		int32 length = object->contents.prop.value.stranger.length;
		char *ptr = object->contents.prop.value.stranger.contents;
	    
		(void) sprintf(buffer,"%D bytes of stranger data:", length);
		while (length-- > 0) {
		    (void) sprintf(buffer,"%s %x ", buffer, *(ptr++));
		}
		(void) sprintf(buffer,"%s\n", buffer);
		vemMessage(buffer, MSG_DISP);
		break;
	    }
	case OCT_ID :
	    (void) sprintf(buffer,"Object id %d\n", object->contents.prop.value.id);
	    vemMessage(buffer, MSG_DISP);
	    break;
	}
	break;
    default :
	(void) sprintf(buffer, "An object of type %s\n",
		oct_type_names[object->type]);
	vemMessage(buffer, MSG_DISP);
	break;
    }
    
    if (octInitGenContentsSpecial(object, OCT_ALL_MASK, &gen) != OCT_OK) {
	oct_prepend_error("rpcPrintObject: generating contents of object");
	return OCT_ERROR;
    }
    
    status = octGenerate(&gen,&sub_object);
    
    if (status == OCT_OK) {
	(void) sprintf(buffer,"    containing ");
	do {
	    (void) sprintf(buffer, "%s %d ", buffer, sub_object.objectId);
	} while ((status = octGenerate(&gen,&sub_object)) == OCT_OK);
	(void) sprintf(buffer, "%s\n", buffer);
	vemMessage(buffer, MSG_DISP);
    }
    if (status != OCT_GEN_DONE) {
	oct_prepend_error("rpcPrintObject: generating contents of object");
	return OCT_ERROR;
    }
    
    if (octInitGenContainersSpecial(object, OCT_ALL_MASK, &gen) != OCT_OK) {
	oct_prepend_error("rpcPrintObject: generating container of object");
	return OCT_ERROR;
    }
    
    status = octGenerate(&gen,&sub_object);
    
    if (status == OCT_OK) {
	(void) sprintf(buffer,"    attached to ");
	do {
	    (void) sprintf(buffer, "%s %d ", buffer, sub_object.objectId);
	} while ((status = octGenerate(&gen,&sub_object)) == OCT_OK);
	(void) sprintf(buffer, "%s\n", buffer);
	vemMessage(buffer, MSG_DISP);
    }
    if (status != OCT_GEN_DONE) {
	oct_prepend_error("rpcPrintObject: generating containers of object");
	return OCT_ERROR;
    }
    return OCT_OK;
}


int
octReadTestExit(spot, cmdList, userOptionWord)
struct RPCSpot *spot;
lsList cmdList;
long userOptionWord;
{
    RPCExit(0);
}


RPCFunction CommandArray[] = {
    {octReadTest, "Oct-Read-Test", "TextDump", "P", 0},
    {octReadTestExit, "Oct-Read-Test", "Exit", "Q", 0}
};


int
UserMain(display, spot, cmdlist, uow, array)
char *display;
struct RPCSpot *spot;
lsList cmdlist;
long uow;
struct RPCFunction **array;
{
    *array = CommandArray;

    VEMMSG("ReadTest initialized\n");
    vemPrompt();
    
    return(sizeof(CommandArray)/sizeof(struct RPCFunction));
}
